<?php

namespace Drupal\imce_dfp\Entity;

use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Plugin\DefaultPluginManager;
use \Drupal\imce\Entity\ImceProfile;
use Drupal\imce_dfp\Service\ImceDfpPluginInterface;

class ImceDfpProfile extends ImceProfile {


  const PLUGIN_KEYWORD = 'dfp_plugin: ';

  /**
   * Returns configuration options.
   */
  public function getConf($key = NULL, $default = NULL) {
    $conf = parent::getConf($key, $default);
    $conf = $this->dynamicPathProcessor($conf);
    return $conf;
  }

  /**
   *  Check imce paths in profile and apply dfp_plugin if needed.
   *
   * @param $profile_id
   *    Imce profile id.
   * @param $conf
   *    Configuration of processed profile.
   * @return mixed
   */
  protected function dynamicPathProcessor ($conf) {
    if (isset($conf['folders']) && is_array($conf['folders']) && !empty($conf['folders'])) {
      // Check for all folder path if there is a plugin keyword.
      foreach($conf['folders']  as $key => $folder) {
        if (isset($folder['path']) && is_string($folder['path'])) {
          $path = $folder['path'];
          $permissions = $folder['permissions'];
          if(substr($path, 0, 12) === self::PLUGIN_KEYWORD) {
            $plugin_id = str_replace(self::PLUGIN_KEYWORD, '', $path);
            $plugin_id = trim($plugin_id);

            // If a plugin_id was found, try to load it.
            if (!empty($plugin_id)) {
              /** @var DefaultPluginManager $imce_dfp_manager */
              $imce_dfp_manager = \Drupal::service('plugin.manager.imce_dfp.path');

              // remove current bad folder path.
              unset($conf['folders'][$key]);

              // Get available folder paths from the loaded plugin.
              try {
                /** @var ImceDfpPluginInterface $imce_dfp_plugin */
                $imce_dfp_plugin = $imce_dfp_manager->createInstance($plugin_id);
                $plugin_paths = $imce_dfp_plugin->getPaths($this->id, $conf);
                if (!empty($plugin_paths)) {
                  foreach($plugin_paths as $plugin_path) {
                    $conf['folders'][] = [
                      'path' => $plugin_path,
                      'permissions' => $permissions,
                    ];
                  }
                }
              }
              catch (PluginNotFoundException $e) {
                \Drupal::logger('imce_dfp')->warning($e->getMessage());
              }
            }
          }
        }
      }
    }
    return $conf;
  }

}
